a = [
    1, 2, 3]

b = [
    x + 1
    for x in range(3)]

c = [[[
    1
]]]

d = [[[
    4]]]

e = [[
    1], 2, 3]

def foo(x, y):
    pass

foo(
    a,
    b)

if (a and
        b):
    pass

if (a
        and b):
    pass



match command.split():
# ^ @keyword.conditional
    case ["quit"]:
    # ^ @keyword.conditional
        print("Goodbye!")
        quit_game()
    case ["look"]:
    # ^ @keyword.conditional
        current_room.describe()
    case ["get", obj]:
    # ^ @keyword.conditional
        character.get(obj, current_room)
    case ["go", direction]:
    # ^ @keyword.conditional
        current_room = current_room.neighbor(direction)
    # The rest of your commands go here

match command.split():
# ^ @keyword.conditional
    case ["drop", *objects]:
    # ^ @keyword.conditional
        for obj in objects:
            character.drop(obj, current_room)

match command.split():
# ^ @keyword.conditional
    case ["quit"]: ... # Code omitted for brevity
    case ["go", direction]: pass
    case ["drop", *objects]: pass
    case _:
        print(f"Sorry, I couldn't understand {command!r}")
                                                   # ^^ @@function.macro

match command.split():
# ^ @keyword.conditional
    case ["north"] | ["go", "north"]:
    # ^ @keyword.conditional
        current_room = current_room.neighbor("north")
    case ["get", obj] | ["pick", "up", obj] | ["pick", obj, "up"]:
    # ^ @keyword.conditional
        pass

match = 2
#   ^ @variable
match, a = 2, 3
#   ^ @variable
match: int = secret
#   ^ @variable
x, match: str = 2, "hey, what's up?"
# <- @variable
#   ^ @variable



if (
    True
    or 1
    or False
):
    pass

if (
    True
    or 1
        or False):
    pass

if (True
    or 1
        or False):
    pass

while (
    False
    or 1
    or False
):
    pass

while (
    False
    or 1
        or False):
    pass

while (False
       or 1
       or False):
    pass



class Fields:
    type: str
#   ^^^^ @variable.member

    def __init__(self, type: str, fields: list[int]) -> None:
#                ^^^^ @variable.builtin
#                      ^^^^ @variable.parameter
#                                 ^^^^^^ @variable.parameter
#                                              ^^^ @type.builtin
#                                                       ^^^^ @constant.builtin
        self.fields = fields
#            ^^^^^^ @variable.member
        self.type = type  # this cannot be highlighted correctly by Treesitter
#            ^^^^ @variable.member
        self.__dunderfield__ = None
#            ^^^^^^^^^^^^^^^ @variable.member
        self._FunKyFielD = 0
#            ^^^^^^^^^^^ @variable.member
        self.NOT_A_FIELD = "IM NOT A FIELD"
#            ^^^^^^^^^^^ @constant

Fields(type="schema", fields=[0, 1])
#      ^^^^ @variable.parameter
#                     ^^^^^^ @variable.parameter



def a():
    return

def a():
    return True

def a():
    return (1, 2, 3)

def a():
    return x.y.z

def a():
    return (
        1, 2, 3
    )

def a():
    return b(
        1, 2, 3
    )

def a():
    return [1, 2, 3]

def a():
    return {1, 2, 3}

def a():
    return {
        "a": 1,
        "b": 2,
        "c": 3
    }

def a():
    return [
        a for a in range (1, 3)
    ]



def func() -> None: ...

_ = func()
#   ^^^^ @function.call

"{}".format(1)
#    ^^^^^^ @function.method.call

class Foo:
    def method(self) -> None: ...
#              ^^^^ @variable.builtin

    @classmethod
    def clsmethod(cls) -> None: ...
#                 ^^^ @variable.builtin

Foo().method()
#     ^^^^^^ @function.method.call

print()
# ^ @function.builtin



# Docstrings according to PEP 257 (https://peps.python.org/pep-0257/)
# <- @comment

"""Module docstring assigned to `__doc__`..."""
# <- @string.documentation
"""
... with an addtional docstring, not part of `__doc__`.
"""
# <- @string.documentation

"""
Some random docstring in the middle if nowhere...
"""
# <- @string.documentation
"""
... also with not one ...
"""
# <- @string.documentation
"""
... but two addtional docstrings.
"""
# <- @string.documentation

oneline_string_assignment = "not detected as docstring"
#                           ^^^^^^^^^^^^^^^^^^^^^^^^^^^ @string
#                           ^^^^^^^^^^^^^^^^^^^^^^^^^^^ !@string.documentation
"""Module attribute docstring."""
# <- @string.documentation

multiline_string_assignment = """
                              also not detected as docstring
                              """
#                             ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ @string
#                             ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ !@string.documentation

looks_like_implicit_string_concatenation = "abc"
#                                          ^^^^^ @string 
"def"
# <- @string.documentation

single_line_implicit_string_concatenation = "abc" "def"
#                                           ^^^^^ @string 
#                                                 ^^^^^ @string 
#                                                 ^^^^^ !@string.documentation 

multiline_implicit_string_concatenation = (
    "not "
    # <- @string
    # <- !@string.documentation
    "detected "
    # <- @string
    # <- !@string.documentation
    "as docstring, "
    # <- @string
    # <- !@string.documentation
    "either."
    # <- @string
    # <- !@string.documentation
)


class A:
    """
    Class docstring, assigned to `__doc__`.
    """
    # <- @string.documentation

    """
    Some random docstring again, ...
    """
    # <- @string.documentation
    """
    ... with an "additional" docstring. Again.
    """
    # <- @string.documentation

    foo = "class attribute"
    #     ^^^^^^^^^^^^^^^^^ @string
    #     ^^^^^^^^^^^^^^^^^ !@string.documentation
    """
    Class attribute docstring, but an attribute 
    does not have a `__doc__` attribute itself.
    """
    # <- @string.documentation

    bar: int
    """Class attribute docstring, type annotation only."""
    # <- @string.documentation

    baz: str = "type annotated class attribute"
    #          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ @string
    #          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ !@string.documentation
    """Class attribute docstring, type annotation and assignment."""
    # <- @string.documentation

    def __init__(self):
        """Method docstring."""
        # <- @string.documentation

        self.quux = "instance attribute"
        #           ^^^^^^^^^^^^^^^^^^^^ @string
        #           ^^^^^^^^^^^^^^^^^^^^ !@string.documentation
        """Instance attribute docstring."""
        # <- @string.documentation


def f(x):
    """Function docstring."""
    # <- @string.documentation
    """Addtional function docstring."""
    # <- @string.documentation
    return x**2


f.a = 1
"""Function attribute docstring."""
# <- @string.documentation


"Random docstring with single quotes - legal, but far off standard and confusing."
# <- @string.documentation



def aligned_indent(arg1nsdkjdsnsdnfsadnnnnfsn,
                   arg2):
    pass

aligned_indent(1,
               2)


aligned_indent(1,
               2
               )

foodsadsa(sdada,
          2



def aligned_indent(arg1,
                   arg2):
    pass

def aligned_indent2(
    arg1,
    arg2
):
    pass

aligned_indent(1,
               2)


aligned_indent(1,
               2
               )

aligned_indent(
    1,
    2
)

foodsadsa(sdada,
          2



(
    a, 
    b
)

foo.bar(
    a, b
)

foo = [
    1, 
    2, 
    3
]

foo = {
    "a": 1,
    "b": 2,
    "c": 3
}

foo = {
    1,
    2,
    3,
}

foo = (
    1 + 2
)

(
    a for a in range(0, 10)
)

foo = [
    a for a in range(0, 10)
]

foo = {
    a for a in range(0, 10)
}

foo = {
    a: b for a, b in items
}

foo.bar(
    "baz")

[
    a + b for (
        a, 
        b
    ) 
    in items
]

[
    a + b for [
        a,
        b
    ]
    in items
]



if a == a:
    x = 1
elif b:
    x = 2
else:
    x = 3

while False:
    pass

for _ in range(3):
    pass

with open('/tmp/f', 'w') as f:
    pass

try:
    pass
except:
    pass
finally:
    pass

while (a > 4 and
       b > 5):
    pass

try:

def foo():
    print('indentme')

    # comment
    if True:
        pass



from dataclasses import dataclass


@dataclass
#^^^^^^^^^ @attribute
class Data:
    _foo: str

    @property
#   ^ @attribute
#    ^^^^^^^^ @attribute.builtin
    def foo(self) -> str:
        return self._foo


@pytest.mark.filterwarnings("ignore::DeprecationWarning")
#^^^^^^ @variable
#       ^^^^ @variable.member
#            ^^^^^^^^^^^^^^ @attribute
def test_func():
    pass




# list
a = [
    x + 1 for x in range(3)
]

# dict
b = {
    x: x + 1 for x in range(3)
}

# generator
c = (
    x * x for x in range(3)
)

# set
d = {
    x + x for x in range(3)
}

# other styles
e = [
    x + 1 for x
    in range(3)
]



import logging
import os
import os.path

from autogit.data_types import CliArguments, RepoState
from autogit.utils.helpers import (
    get_domain,
    get_repo_group,
    get_repo_name,
    get_repo_owner,
    to_kebab_case,
)

logger = logging.getLogger()


def is_url_or_git(file_names_or_repo_url: str) -> bool:
    # TODO: use urlparse to verify if its url and use regexp for git url
    return '.com' in file_names_or_repo_url.lower()


def read_repositories_from_file(repos_filename: str) -> list[str]:
    """Reads a list of repositories from a file while ignoring commented out lines."""
    with open(repos_filename) as f:
        return [line.strip() for line in f if not line.strip().startswith('#')]


def get_repository_state(repo_url: str, branch: str, args: CliArguments) -> RepoState:
    repo_name = get_repo_name(repo_url)
    repo_owner = get_repo_owner(repo_url)
    repo_group = get_repo_group(repo_url)
    domain = get_domain(repo_url)

    return RepoState(
        args=args,
        name=repo_name,
        owner=repo_owner,
        group=repo_group,
        url=repo_url,
        domain=domain,
        branch=branch,
    )


def get_repository_states(args: CliArguments) -> dict[str, RepoState]:
    repo_urls = []
    for file_names_or_repo_url in args.repos:
        if not is_url_or_git(file_names_or_repo_url) and os.path.exists(file_names_or_repo_url):
            newly_read_repos = read_repositories_from_file(file_names_or_repo_url)
            repo_urls.extend(newly_read_repos)
        else:
            repo_urls.append(file_names_or_repo_url)

    branch = args.branch or to_kebab_case(args.commit_message)

    repos: dict[str, RepoState] = {}
    for repo_url in repo_urls:
        repo_state = get_repository_state(repo_url, branch, args)
        repos[repo_state.name] = repo_state

    return repos
